package info.bukova.isspst.services.approved;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;

import info.bukova.isspst.data.AddressEmb;
import info.bukova.isspst.data.JoinedItem;
import info.bukova.isspst.data.Order;
import info.bukova.isspst.data.OrderItem;
import info.bukova.isspst.services.AbstractOwnedService;
import info.bukova.isspst.services.settings.GlobalSettingsService;

public class OrderServiceImpl extends AbstractOwnedService<Order> implements
		OrderService {

	@Autowired
	private GlobalSettingsService globalSettings;
	
	@Override
	@PreAuthorize("hasPermission(this, 'PERM_ADD')")
	public Order createOrder(List<JoinedItem> items) {
		Order order = new Order();
		
		order.setAddress(new AddressEmb(globalSettings.getSettings().getMainAddress()));
		order.setDeliveryAddress(new AddressEmb(globalSettings.getSettings().getMainAddress()));
		order.setOrderDate(new Date());
		
		BigDecimal total = BigDecimal.ZERO;
		
		for (JoinedItem item : items) {
			total = total.add(item.getTotal());
			order.addItem(new OrderItem(item.getItem()));
		}
		
		order.setTotal(total);
		
		return order;
	}
	
	@Override
	@Transactional
	@PreAuthorize("hasPermission(this, 'PERM_ADD')")
	public void add(Order order) {
		order.setNumser(getNumberSerie());
		super.add(order);
	}

}
